"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetupOpenVpn = void 0;
const chalk_1 = __importDefault(require("chalk"));
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const package_json_1 = __importDefault(require("../package.json"));
const shelljs_1 = __importDefault(require("shelljs"));
const s3_utils_1 = require("./utils/s3-utils");
const fs_utils_1 = require("./utils/fs-utils");
class SetupOpenVpn {
    constructor(options) {
        this.options = options;
        this.s3Client = new aws_sdk_1.default.S3({
            region: this.options.region
        });
        this.run();
    }
    run() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Running openvpn unattended setup ${package_json_1.default.version}`));
            this.setupOpenVpn();
            this.setupSsl();
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Something failed during openvpn setup`), e);
            return;
        }
    }
    setupOpenVpn() {
        const { userName, userPassword, ip, hostName } = this.options;
        if (typeof ip !== 'undefined') {
            console.log(chalk_1.default.bgGreenBright('Setting upstream dns settings and access server hostname'));
            try {
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.client.routing.reroute_dns --value custom ConfigPut`);
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.server.dhcp_option.dns.0 --value ${ip} ConfigPut`);
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.server.routing.gateway_access --value true ConfigPut`);
                shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key host.name --value ${hostName} ConfigPut`);
            }
            catch (e) {
                console.log(chalk_1.default.bgRedBright('An error occured while setting upstream dns settings'), e);
                throw e;
            }
        }
        try {
            console.log(chalk_1.default.bgGreenBright('Creating default client user for OpenVPN'));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --key vpn.client.routing.reroute_gw --value true ConfigPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName} --key type --value user_connect UserPropPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName} --key prop_autologin --value true UserPropPut`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli --user ${userName}  --new_pass ${userPassword} SetLocalPassword`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while creating default client user'), e);
            throw e;
        }
    }
    setupSsl() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(chalk_1.default.bgGreenBright(`Starting process to append an SSL certificate to OpenVPN Access Server Web UI`));
            const { domainName, email } = this.options;
            this.installCertbot();
            const existingCertificate = this.getCertificate();
            if (typeof existingCertificate === 'undefined') {
                console.log(chalk_1.default.yellowBright(`No existing certificate to reuse, creating new cert`));
                try {
                    shelljs_1.default.exec([
                        'sudo',
                        'certbot',
                        'certonly',
                        '--standalone',
                        '--non-interactive',
                        '--agree-tos',
                        '--email',
                        email,
                        '--domains',
                        domainName,
                        ...(this.options.certEnvironment === 'staging'
                            ? [
                                '--server',
                                'https://acme-staging-v02.api.letsencrypt.org/directory'
                            ]
                            : [])
                    ].join(' '));
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright('An error occured while generating a cert'), e);
                    throw e;
                }
                try {
                    shelljs_1.default.exec(`sudo chmod -R 755 /etc/letsencrypt`);
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright('An error while chaging directory permissions'), e);
                    throw e;
                }
            }
            else {
                console.log(chalk_1.default.bgGreenBright(`Existing cert found, reusing it...`));
                try {
                    shelljs_1.default.exec([
                        'sudo',
                        'mkdir',
                        '-p',
                        `/etc/letsencrypt/live/${domainName}`
                    ].join(' '));
                    shelljs_1.default.exec(['sudo', 'chmod', '-R', '777', `/etc/letsencrypt`].join(' '));
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright(`An error occured while creating relevant directories to store certificate`), e);
                    throw e;
                }
                try {
                    const certificateResponse = yield this.getCertificate();
                    if (typeof certificateResponse !== 'undefined') {
                        fs_utils_1.writeToFile(certificateResponse.certPem, `/etc/letsencrypt/live/${domainName}/cert.pem`);
                        fs_utils_1.writeToFile(certificateResponse.privkeyPem, `/etc/letsencrypt/live/${domainName}/privkey.pem.enc`);
                        fs_utils_1.writeToFile(certificateResponse.chainPem, `/etc/letsencrypt/live/${domainName}/chain.pem`);
                        fs_utils_1.writeToFile(certificateResponse.fullchainPem, `/etc/letsencrypt/live/${domainName}/fullchain.pem`);
                    }
                    else {
                        console.log(chalk_1.default.bgYellowBright(`Although initial certificate check showed an existing certificate to use, the actual content of the certificate was not found/usable`));
                    }
                }
                catch (e) {
                    console.log(chalk_1.default.bgRedBright(`An error occured while getting certificate artifacts from S3`), e);
                    throw e;
                }
            }
            this.installCertificate();
            this.startOpenVpn();
        });
    }
    installCertbot() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Installing certbot`));
            shelljs_1.default.exec(`sudo apt-get -y install software-properties-common`);
            shelljs_1.default.exec(`sudo add-apt-repository ppa:certbot/certbot -y`);
            shelljs_1.default.exec(`sudo apt-get -y update`);
            shelljs_1.default.exec(`sudo apt-get -y install certbot`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while installing certbot'), e);
            throw e;
        }
    }
    installCertificate() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Delete existing cert references in OpenVPN DB`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.ca_bundle`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.priv_key`);
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/confdba -mk cs.cert`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while deleting certificate references in OpenVPN DB'), e);
            return;
        }
        try {
            console.log(chalk_1.default.bgGreenBright(`Apply symbolic links to SSL certificates`));
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/cert.pem /usr/local/openvpn_as/etc/web-ssl/server.crt`);
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/privkey.pem /usr/local/openvpn_as/etc/web-ssl/server.key`);
            shelljs_1.default.exec(`sudo ln -s -f /etc/letsencrypt/live/${this.options.domainName}/chain.pem /usr/local/openvpn_as/etc/web-ssl/ca.crt`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured symlinking to certificate files'), e);
            return;
        }
    }
    stopOpenVpn() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Stopping openvpn`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli stop`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while stopping openvpn'), e);
            return;
        }
    }
    startOpenVpn() {
        try {
            console.log(chalk_1.default.blueBright(`Starting openvpn`));
            shelljs_1.default.exec(`sudo /usr/local/openvpn_as/scripts/sacli start`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright('An error occured while starting openvpn'), e);
            return;
        }
    }
    getCertificate() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const checks = yield Promise.all([
                    s3_utils_1.objectExists(this.s3Client, this.options.bucketName, `letsencrypt/${this.options.domainName}/cert.pem`),
                    s3_utils_1.objectExists(this.s3Client, this.options.bucketName, `letsencrypt/${this.options.domainName}/privkey.pem`),
                    s3_utils_1.objectExists(this.s3Client, this.options.bucketName, `letsencrypt/${this.options.domainName}/chain.pem`),
                    s3_utils_1.objectExists(this.s3Client, this.options.bucketName, `letsencrypt/${this.options.domainName}/fullchain.pem`)
                ]);
                const certificateExists = checks.every((value) => value === true);
                if (certificateExists) {
                    const getResults = yield Promise.all([
                        s3_utils_1.getObject(this.s3Client, this.options.bucketName, `letsencrypt/${this.options.domainName}/cert.pem`),
                        s3_utils_1.getObject(this.s3Client, this.options.bucketName, `letsencrypt/${this.options.domainName}/privkey.pem`),
                        s3_utils_1.getObject(this.s3Client, this.options.bucketName, `letsencrypt/${this.options.domainName}/chain.pem`),
                        s3_utils_1.getObject(this.s3Client, this.options.bucketName, `letsencrypt/${this.options.domainName}/fullchain.pem`)
                    ]);
                    const hasContent = getResults.every((value) => typeof value !== 'undefined');
                    if (hasContent) {
                        return {
                            certPem: getResults[0],
                            privkeyPem: getResults[1],
                            chainPem: getResults[2],
                            fullchainPem: getResults[3]
                        };
                    }
                    else {
                        console.log(chalk_1.default.bgYellowBright(`Some or all the required certificate elements did not have any content when fetched from the s3 bucket ${this.options.bucketName}`));
                        return undefined;
                    }
                }
                else {
                    console.log(chalk_1.default.bgYellowBright(`Some or all the required certificate elements could not be found in the s3 bucket ${this.options.bucketName}`));
                    return undefined;
                }
            }
            catch (e) {
                console.log(chalk_1.default.bgRedBright(`Failed to obtain existing certifate`), e);
                return undefined;
            }
        });
    }
}
exports.SetupOpenVpn = SetupOpenVpn;
//# sourceMappingURL=openvpn-unattended-install.js.map